#!/usr/bin/perl -s
##
## Razor::String -- String functions required by Razor.
##
## Copyright (c) 1999, Vipul Ved Prakash.  All rights reserved.
## This code is free software; you can redistribute it and/or modify
## it under the same terms as Perl itself.
##
## $Id: String.pm,v 1.4 2001/10/26 03:13:01 vipul Exp $

package Razor::String; 
use strict;
use Mail::Internet;
use Digest::SHA1;
use Exporter; 
use vars qw ( @ISA $VERSION @EXPORT_OK ); 
@ISA = "Exporter"; 
( $VERSION )  = '$Revision: 1.4 $' =~ /\s(\d+\.\d+)\s/; 

@EXPORT_OK = qw( hash2str str2hash hash ); 
my @ESCAPE = ( '&', ',', ':' );


sub hash2str { 

    my $href = shift; 
    my %hash = %$href; 
    my ($str, $key);

    for $key ( keys %hash ) { 
        my $tstr;
        if ( ref $hash{$key} eq 'ARRAY' ) { 
            for ( @{ $hash{ $key }} )  { $tstr .= escape( $_ ) . "," } $str =~ s/,$//;
        } elsif ( !(ref $hash{$key}) ) { 
            $tstr .= escape ( $hash{$key} );
        }
        if ( $tstr ) { $str .= "$key:$tstr&" }
    }

    $str =~ s/&$//; return $str;

}


sub str2hash { 

    my $str = shift; 
    my %hash;
    my @pairs = split /(?<!\\)&/, $str; 

    for ( @pairs ) { 
        my ( $key, $data ) = split /(?<!\\):/, $_, 2;
        if ( $data =~ /(?<!\\),/ ) { 
            my @list = split /(?<!\\),/, $data; 
            for ( @list ) { $_ = unescape ( $_ ) };
            $hash{$key} = [@list];
        } else { $hash{$key} = unescape ( $data ) } 
    }

    return \%hash; 

}


sub hash { 

    my ( $spam ) = @_; 
    my $mail = new Mail::Internet ( $spam ); 
    $mail->tidy_body(); 
    $mail->remove_sig (); $mail->tidy_body (); 
    my $body = $mail->body();

    # print "--||--\n@$body--||--\n";
    # changes in 0.25
    # my $bodytext = "@$body";
    # $bodytext =~ s/\W//g; 
    # print $bodytext;
    
    my $ctx = Digest::SHA1->new;
    $ctx->add( @$body );
    return $ctx->hexdigest;

}


sub escape { 
    my $str = shift;
    for ( @ESCAPE ) { 
        my $e = $_; $str =~ s/$e/\\$e/g;
    }
    return $str;
}


sub unescape { 
    my $str = shift; 
    for ( @ESCAPE ) {
        my $e = $_; $str =~ s/\\$e/$e/g; 
    }
    return $str; 
}


1;

